@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2002 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,13 +15,22 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         config.cpp  -  description
  *                         --------------------------
  *   begin                : Wed Feb 14 2001
- *   copyright            : (C) 2000-2002 by Damien Sandras 
+ *   copyright            : (C) 2000-2003 by Damien Sandras 
  *   description          : This file contains most of gconf stuff.
  *                          All notifiers are here.
  *                          Callbacks that updates the gconf cache 
@@ -33,25 +42,21 @@
 
 
 #include "../config.h"
+
 #include "config.h"
-#include "common.h"
-#include "callbacks.h"
-#include "sound_handling.h"
-#include "videograbber.h"
+#include "connection.h"
 #include "gnomemeeting.h"
-#include "misc.h"
+#include "videograbber.h"
+#include "ils.h"
+#include "urlhandler.h"
+#include "sound_handling.h"
 #include "pref_window.h"
 #include "ldap_window.h"
-#include "main_window.h"
-#include "ils.h"
-#include "dialog.h"
-#include "menu.h"
 #include "tray.h"
-#include "lid.h"
-
-#ifndef DISABLE_GNOME
-#include <gnome.h>
-#endif
+#include "misc.h"
+#include "menu.h"
+#include "dialog.h"
+#include "stock-icons.h"
 
 
 /* Declarations */
@@ -91,17 +96,20 @@ static void video_preview_changed_nt (GConfClient *, guint, GConfEntry *,
 				      gpointer);
 static void audio_codecs_list_changed_nt (GConfClient *, guint, GConfEntry *, 
 					  gpointer);
-static void contacts_list_group_content_changed_nt (GConfClient *,
-						    guint, GConfEntry *, 
-						    gpointer);
-static void contacts_list_changed_nt (GConfClient *, guint, GConfEntry *, 
-				      gpointer);
+static void contacts_sections_list_group_content_changed_nt (GConfClient *,
+							     guint, 
+							     GConfEntry *, 
+							     gpointer);
+static void contacts_sections_list_changed_nt (GConfClient *, guint, 
+					       GConfEntry *, gpointer);
 static void view_widget_changed_nt (GConfClient *, guint, GConfEntry *, 
 				    gpointer);
 static void capabilities_changed_nt (GConfClient *, guint, 
 				     GConfEntry *, gpointer);
+#ifndef DISABLE_GNOME
 static void microtelco_enabled_nt (GConfClient *, guint, GConfEntry *,
 				   gpointer);
+#endif
 static void ht_fs_changed_nt (GConfClient *, guint, GConfEntry *, gpointer);
 static void enable_vid_tr_changed_nt (GConfClient *, guint, GConfEntry *, 
 				      gpointer);
@@ -110,8 +118,6 @@ static void silence_detection_changed_nt (GConfClient *, guint,
 static void network_settings_changed_nt (GConfClient *, guint, 
 					 GConfEntry *, gpointer);
 #ifdef HAS_IXJ
-static void lid_device_changed_nt (GConfClient *, guint, GConfEntry *, 
-				   gpointer);
 static void lid_aec_changed_nt (GConfClient *, guint, GConfEntry *, gpointer);
 static void lid_country_changed_nt (GConfClient *, guint, GConfEntry *, 
 				    gpointer);
@@ -400,7 +406,7 @@ static void applicability_check_nt (GConfClient *client, guint cid,
     gdk_threads_enter ();
   
     if (MyApp->Endpoint ()->GetCallingState () != 0)
-      gnomemeeting_warning_dialog_on_widget (GTK_WINDOW (gm), GTK_WIDGET (data), _("Changing this setting will only affect new calls"));
+      gnomemeeting_warning_dialog_on_widget (GTK_WINDOW (gm), GTK_WIDGET (data), _("Changing this setting will only affect new calls"), _("You have changed a setting that doesn't permit to GnomeMeeting to apply the new change to the current call. Your new setting will only take effect for the next call."));
     
     gdk_threads_leave ();
   }
@@ -422,7 +428,7 @@ static void main_notebook_changed_nt (GConfClient *client, guint cid,
 
     gdk_threads_enter ();
 
-    gw = gnomemeeting_get_main_window (gm);
+    gw = MyApp->GetMainWindow ();
 
     if (gconf_value_get_int (entry->value) == GM_MAIN_NOTEBOOK_HIDDEN)
       gtk_widget_hide_all (gw->main_notebook);
@@ -445,6 +451,7 @@ static void main_notebook_changed_nt (GConfClient *client, guint cid,
  *                 also updates the ixj druid page.
  * PRE          :  /
  */
+#ifndef DISABLE_GNOME
 static void microtelco_enabled_nt (GConfClient *client, guint cid, 
 				   GConfEntry *entry, gpointer data)
 {
@@ -456,7 +463,7 @@ static void microtelco_enabled_nt (GConfClient *client, guint cid,
     gdk_threads_enter ();
 
     gnomemeeting_menu = gnomemeeting_get_menu (gm);
-    dw = gnomemeeting_get_druid_window (gm);
+    dw = MyApp->GetDruidWindow ();
     
     if (gconf_value_get_bool (entry->value)) {
 
@@ -475,6 +482,7 @@ static void microtelco_enabled_nt (GConfClient *client, guint cid,
     gdk_threads_leave ();
   }
 }
+#endif
 
 
 /* DESCRIPTION  :  This notifier is called when the gconf database data
@@ -531,43 +539,70 @@ static void enable_vid_tr_changed_nt (GConfClient *client, guint cid,
 static void silence_detection_changed_nt (GConfClient *client, guint cid, 
 					  GConfEntry *entry, gpointer data)
 {
+  H323Codec *raw_codec = NULL;
+  H323Connection *connection = NULL;
+  H323Channel *channel = NULL;
   H323AudioCodec *ac = NULL;
+  GMH323EndPoint *endpoint = NULL;
+  
+  PString current_call_token;
+  
   GmWindow *gw = NULL;
-
+  endpoint = MyApp->Endpoint ();
+  
   if (entry->value->type == GCONF_VALUE_BOOL) {
 
     gdk_threads_enter ();
 
-    /* We update the silence detection */
-    if (MyApp->Endpoint ()->GetCallingState () == 2) {
+    current_call_token = endpoint->GetCurrentCallToken ();
 
-      ac = MyApp->Endpoint ()->GetCurrentAudioCodec ();
-      gw = gnomemeeting_get_main_window (gm);
+    if (!current_call_token.IsEmpty ())
+      connection =
+	endpoint->GetCurrentConnection ();
 
-      if (ac != NULL) {
+    if (connection) {
 
-	H323AudioCodec::SilenceDetectionMode mode = 
-	  ac->GetSilenceDetectionMode();
-    
-	if (mode == H323AudioCodec::AdaptiveSilenceDetection) {
+      channel = 
+	connection->FindChannel (RTP_Session::DefaultAudioSessionID, 
+				 FALSE);
+
+      if (channel)
+	raw_codec = channel->GetCodec();
+      
+      if (raw_codec && raw_codec->IsDescendant (H323AudioCodec::Class())) {
+
+	ac = (H323AudioCodec *) raw_codec;
+      }
+   
+      /* We update the silence detection */
+      if (ac && MyApp->Endpoint ()->GetCallingState () == 2) {
+	
+	gw = MyApp->GetMainWindow ();
+	
+	if (ac != NULL) {
 	  
-	  mode = H323AudioCodec::NoSilenceDetection;
-	  gnomemeeting_log_insert (gw->history_text_view,
-				   _("Disabled Silence Detection"));
-	} 
-	else {
-
-	  mode = H323AudioCodec::AdaptiveSilenceDetection;
-	  gnomemeeting_log_insert (gw->history_text_view,
-				   _("Enabled Silence Detection"));
+	  H323AudioCodec::SilenceDetectionMode mode = 
+	    ac->GetSilenceDetectionMode();
+	  
+	  if (mode == H323AudioCodec::AdaptiveSilenceDetection) {
+	    
+	    mode = H323AudioCodec::NoSilenceDetection;
+	    gnomemeeting_log_insert (gw->history_text_view,
+				     _("Disabled Silence Detection"));
+	  } 
+	  else {
+	    
+	    mode = H323AudioCodec::AdaptiveSilenceDetection;
+	    gnomemeeting_log_insert (gw->history_text_view,
+				     _("Enabled Silence Detection"));
+	  }
+	  
+	  ac->SetSilenceDetectionMode(mode);
 	}
-
-	ac->SetSilenceDetectionMode(mode);
       }
     }
 
-    gdk_threads_leave ();
-
+    gdk_threads_leave ();  
   }
 }
 
@@ -600,7 +635,16 @@ static void capabilities_changed_nt (GConfClient *client, guint i,
 static void fps_limit_changed_nt (GConfClient *client, guint cid, 
 				  GConfEntry *entry, gpointer data)
 {
+  H323Connection *connection = NULL;
+  H323Channel *channel = NULL;
+  H323Codec *raw_codec = NULL;
   H323VideoCodec *vc = NULL;
+  GMH323EndPoint *endpoint = NULL;
+
+  PString current_call_token;
+
+  endpoint = MyApp->Endpoint ();
+  
   int fps = 30;
   double frame_time = 0.0;
 
@@ -608,16 +652,36 @@ static void fps_limit_changed_nt (GConfClient *client, guint cid,
 
     gdk_threads_enter ();
   
-    /* We update the minimum fps limit */
-    vc = MyApp->Endpoint ()->GetCurrentVideoCodec ();
-  
-    fps = gconf_value_get_int (entry->value);
-    frame_time = (unsigned) (1000.0 / fps);
-    frame_time = PMAX (33, PMIN(1000000, frame_time));
+    current_call_token = endpoint->GetCurrentCallToken ();
 
-    if (vc != NULL)
-      vc->SetTargetFrameTimeMs ((unsigned int) frame_time);
+    if (!current_call_token.IsEmpty ())
+      connection =
+	endpoint->GetCurrentConnection ();
 
+    if (connection) {
+
+      channel = 
+	connection->FindChannel (RTP_Session::DefaultVideoSessionID, 
+				 FALSE);
+
+      if (channel)
+	raw_codec = channel->GetCodec();
+      
+      if (raw_codec && raw_codec->IsDescendant (H323VideoCodec::Class())) {
+
+	vc = (H323VideoCodec *) raw_codec;
+      }
+   
+
+      /* We update the minimum fps limit */
+      fps = gconf_value_get_int (entry->value);
+      frame_time = (unsigned) (1000.0 / fps);
+      frame_time = PMAX (33, PMIN(1000000, frame_time));
+
+      if (vc != NULL)
+	vc->SetTargetFrameTimeMs ((unsigned int) frame_time);
+    }
+    
     gdk_threads_leave ();
   }
 }
@@ -632,22 +696,51 @@ static void
 maximum_video_bandwidth_changed_nt (GConfClient *client, guint cid, 
 				    GConfEntry *entry, gpointer data)
 {
+  H323Channel *channel = NULL;
+  H323Codec *raw_codec = NULL;
   H323VideoCodec *vc = NULL;
+  H323Connection *connection = NULL;
+  GMH323EndPoint *endpoint = NULL;
+
+  PString current_call_token;
+
   int bitrate = 2;
 
+  endpoint = MyApp->Endpoint ();
+  
 
   if (entry->value->type == GCONF_VALUE_INT) {
 
     gdk_threads_enter ();
+
+    current_call_token = endpoint->GetCurrentCallToken ();
+
+    if (!current_call_token.IsEmpty ())
+      connection =
+	endpoint->GetCurrentConnection ();
+
+    if (connection) {
+
+      channel = 
+	connection->FindChannel (RTP_Session::DefaultVideoSessionID, 
+				 FALSE);
+
+      if (channel)
+	raw_codec = channel->GetCodec();
+      
+      if (raw_codec && raw_codec->IsDescendant (H323VideoCodec::Class())) {
+
+	vc = (H323VideoCodec *) raw_codec;
+      }
+
+      /* We update the video quality */  
+      bitrate = gconf_value_get_int (entry->value) * 8 * 1024;
   
-    /* We update the video quality */
-    vc = MyApp->Endpoint ()->GetCurrentVideoCodec ();
-  
-    bitrate = gconf_value_get_int (entry->value) * 8 * 1024;
-  
-    if (vc != NULL)
-      vc->SetMaxBitRate (bitrate);
-  
+      if (vc != NULL)
+	vc->SetMaxBitRate (bitrate);
+
+    }
+    
     gdk_threads_leave ();
   }
 }
@@ -660,22 +753,50 @@ maximum_video_bandwidth_changed_nt (GConfClient *client, guint cid,
 static void tr_vq_changed_nt (GConfClient *client, guint cid, 
 			      GConfEntry *entry, gpointer data)
 {
+  H323Connection *connection = NULL;
+  H323Channel *channel = NULL;
+  H323Codec *raw_codec = NULL;
   H323VideoCodec *vc = NULL;
+  GMH323EndPoint *endpoint = NULL;
+
+  PString current_call_token;
+  
   int vq = 1;
 
+  endpoint = MyApp->Endpoint ();
 
   if (entry->value->type == GCONF_VALUE_INT) {
 
     gdk_threads_enter ();
-  
-    /* We update the video quality */
-    vc = MyApp->Endpoint ()->GetCurrentVideoCodec ();
 
-    vq = 25 - (int) ((double) (int) gconf_value_get_int (entry->value) / 100 * 24);
-  
-    if (vc != NULL)
-      vc->SetTxMaxQuality (vq);
+    current_call_token = endpoint->GetCurrentCallToken ();
+
+    if (!current_call_token.IsEmpty ())
+      connection =
+	endpoint->GetCurrentConnection ();
+
+    if (connection) {
+
+      channel = 
+	connection->FindChannel (RTP_Session::DefaultVideoSessionID, 
+				 FALSE);
+
+      if (channel)
+	raw_codec = channel->GetCodec();
+      
+      if (raw_codec && raw_codec->IsDescendant (H323VideoCodec::Class())) {
+
+	vc = (H323VideoCodec *) raw_codec;
+      }
+
+      /* We update the video quality */
+      vq = 25 - (int) ((double) (int) gconf_value_get_int (entry->value) / 100 * 24);
   
+      if (vc != NULL)
+	vc->SetTxMaxQuality (vq);
+    }
+
+    
     gdk_threads_leave ();
   }
 }
@@ -688,17 +809,44 @@ static void tr_vq_changed_nt (GConfClient *client, guint cid,
 static void tr_ub_changed_nt (GConfClient *client, guint cid, 
 			      GConfEntry *entry, gpointer data)
 {
+  H323Connection *connection = NULL;
+  H323Channel *channel = NULL;
+  H323Codec *raw_codec = NULL;
   H323VideoCodec *vc = NULL;
+  GMH323EndPoint *endpoint = NULL;
+
+  PString current_call_token;
+  
+  endpoint = MyApp->Endpoint ();
 
   if (entry->value->type == GCONF_VALUE_INT) {
 
     gdk_threads_enter ();
 
-    /* We update the current tr ub rate */
-    vc = MyApp->Endpoint ()->GetCurrentVideoCodec ();
-    
-    if (vc != NULL)
-      vc->SetBackgroundFill ((int) gconf_value_get_int (entry->value));
+        current_call_token = endpoint->GetCurrentCallToken ();
+
+    if (!current_call_token.IsEmpty ())
+      connection =
+	endpoint->GetCurrentConnection ();
+
+    if (connection) {
+
+      channel = 
+	connection->FindChannel (RTP_Session::DefaultVideoSessionID, 
+				 FALSE);
+
+      if (channel)
+	raw_codec = channel->GetCodec();
+      
+      if (raw_codec && raw_codec->IsDescendant (H323VideoCodec::Class())) {
+
+	vc = (H323VideoCodec *) raw_codec;
+      }
+
+      /* We update the current tr ub rate */
+      if (vc != NULL)
+	vc->SetBackgroundFill ((int) gconf_value_get_int (entry->value));
+    }
     
     gdk_threads_leave ();
   }
@@ -724,7 +872,7 @@ static void jitter_buffer_changed_nt (GConfClient *client, guint cid,
 
     gdk_threads_enter ();
 
-    pw = gnomemeeting_get_pref_window (gm);
+    pw = MyApp->GetPrefWindow ();
 
     min_val = 
       gconf_client_get_int (client, AUDIO_SETTINGS_KEY "min_jitter_buffer", 0);
@@ -734,7 +882,7 @@ static void jitter_buffer_changed_nt (GConfClient *client, guint cid,
 
     g_signal_handlers_block_by_func (G_OBJECT (data),
 				     (gpointer) adjustment_changed, 
-				       (gpointer) g_object_get_data (G_OBJECT (data), 
+				     (gpointer) g_object_get_data (G_OBJECT (data), 
 								     "gconf_key")); 
 
     if (data == pw->max_jitter_buffer)
@@ -785,16 +933,16 @@ audio_mixer_changed_nt (GConfClient *client,
   GMLid *lid = NULL;
 #endif
   
-  if (entry->value->type == GCONF_VALUE_STRING) {
+  if (entry->value->type == GCONF_VALUE_STRING && endpoint) {
 
 #ifdef HAS_IXJ
-    lid = endpoint->GetLidThread ();
+    lid = endpoint->GetLid ();
 
     if (!lid) {
 #endif
       gdk_threads_enter ();
 
-      gw = gnomemeeting_get_main_window (gm);
+      gw = MyApp->GetMainWindow ();
       mixer = (char *) gconf_value_get_string (entry->value);
 
       vol = gnomemeeting_get_mixer_volume (mixer, GPOINTER_TO_INT (data));
@@ -837,9 +985,9 @@ static void audio_device_changed_nt (GConfClient *client, guint cid,
   if (entry->value->type == GCONF_VALUE_STRING) {
 
     gdk_threads_enter ();
-    dw = gnomemeeting_get_druid_window (gm);
-    pw = gnomemeeting_get_pref_window (gm);
-    gw = gnomemeeting_get_main_window (gm);
+    dw = MyApp->GetDruidWindow ();
+    pw = MyApp->GetPrefWindow ();
+    gw = MyApp->GetMainWindow ();
       
     dev = PString (gconf_value_get_string (entry->value));
 
@@ -896,48 +1044,6 @@ static void audio_device_changed_nt (GConfClient *client, guint cid,
 }
 
 
-#ifdef HAS_IXJ
-/* DESCRIPTION  :  This notifier is called when the gconf database data
- *                 associated with the lid device changes.
- * BEHAVIOR     :  It updates the endpoint and displays
- *                 a message in the history. If the device is not valid,
- *                 a message is displayed. Disable Speaker Phone mode, and
- *                 show/hide the toolbar button for speaker phone following
- *                 Quicknet is used or not.
- * PRE          :  /
- */
-static void lid_device_changed_nt (GConfClient *client, guint cid, 
-				   GConfEntry *entry, gpointer data)
-{
-  GmWindow *gw = NULL;
-
-  if (entry->value->type == GCONF_VALUE_BOOL) {
-
-    gdk_threads_enter ();
-    gw = gnomemeeting_get_main_window (gm);
-    
-    if (MyApp->Endpoint ()->GetCallingState () == 0) {
-
-      /* Update the configuration in order to update 
-	 the local user name for calls */
-      MyApp->Endpoint ()->UpdateConfig ();
-      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gw->speaker_phone_button),
-				   FALSE);
-    }
-
-    /*
-    if (gconf_value_get_bool (entry->value))
-      gtk_widget_show_all (gw->speaker_phone_button);
-    else
-      gtk_widget_hide_all (gw->speaker_phone_button);
-    */
-
-    gdk_threads_leave ();
-  }
-}
-#endif
-
-
 /* DESCRIPTION  :  This callback is called when the video device changes in
  *                 the gconf database.
  * BEHAVIOR     :  It resets the video device.
@@ -946,6 +1052,7 @@ static void lid_device_changed_nt (GConfClient *client, guint cid,
 static void video_device_setting_changed_nt (GConfClient *client, guint cid, 
 					     GConfEntry *entry, gpointer data)
 {
+  GMH323EndPoint *ep = NULL;
   GMVideoGrabber *vg = NULL;
 
   if ((entry->value->type == GCONF_VALUE_STRING) ||
@@ -954,12 +1061,17 @@ static void video_device_setting_changed_nt (GConfClient *client, guint cid,
     gdk_threads_enter ();
 
     /* We reset the video device */
-    if (MyApp->Endpoint ()->GetCallingState () == 0) {
+    ep = MyApp->Endpoint ();
     
-      vg = GM_VIDEO_GRABBER (MyApp->Endpoint ()->GetVideoGrabberThread ());
+    if (ep && ep->GetCallingState () == 0) {
+    
+      vg = ep->GetVideoGrabber ();
 
-      if (vg)
+      if (vg) {
+	
 	vg->Reset ();
+	vg->Unlock ();
+      }
     }
   
     gdk_threads_leave ();
@@ -974,27 +1086,21 @@ static void video_device_setting_changed_nt (GConfClient *client, guint cid,
 static void video_preview_changed_nt (GConfClient *client, guint cid, 
 				      GConfEntry *entry, gpointer data)
 {
+  GMH323EndPoint *ep = NULL;
+  
   if (entry->value->type == GCONF_VALUE_BOOL) {
    
     gdk_threads_enter ();
-
-    GMVideoGrabber *vg = NULL;
          
     /* We reset the video device */
-    if (MyApp->Endpoint ()->GetCallingState () == 0) {
+    ep = MyApp->Endpoint ();
     
-      vg = GM_VIDEO_GRABBER (MyApp->Endpoint ()->GetVideoGrabberThread ());
+    if (ep && ep->GetCallingState () == 0) {
     
-      if (gconf_value_get_bool (entry->value)) {
-
-	if (!vg->IsOpened ())
-	  vg->Open (TRUE);
-      }
-      else {
-      
-	if (vg->IsOpened ())
-	  vg->Close ();
-      }
+      if (gconf_value_get_bool (entry->value)) 
+	ep->CreateVideoGrabber ();
+      else 
+	ep->RemoveVideoGrabber ();
     }
  
     gdk_threads_leave ();
@@ -1017,7 +1123,7 @@ static void audio_codecs_list_changed_nt (GConfClient *client, guint cid,
    
     gdk_threads_enter ();
 
-    pw = gnomemeeting_get_pref_window (gm);
+    pw = MyApp->GetPrefWindow ();
 
     /* We set the new value for the widget */
     gnomemeeting_codecs_list_build (pw->codecs_list_store);
@@ -1033,27 +1139,28 @@ static void audio_codecs_list_changed_nt (GConfClient *client, guint cid,
 
 
 static void
-contacts_list_group_content_changed_nt (GConfClient *client, guint cid,
-					GConfEntry *e, gpointer data)
+contacts_sections_list_group_content_changed_nt (GConfClient *client, 
+						 guint cid,
+						 GConfEntry *e, gpointer data)
 {
   const char *gconf_key = NULL;
   gchar **group_split = NULL;
   gchar *group_name = NULL;
-  gchar *contact_section = NULL;
-
-  GtkListStore *list_store = NULL;
+  gchar *group_name_unescaped = NULL;
   
   int cpt = 0;
 
   GtkWidget *page = NULL;
-
+  GtkListStore *list_store = NULL;
+  
   GmLdapWindow *lw = NULL;
+  GmLdapWindowPage *lwp = NULL;
   
   if (e->value->type == GCONF_VALUE_LIST) {
   
     gdk_threads_enter ();
 
-    lw = gnomemeeting_get_ldap_window (gm);
+    lw = MyApp->GetLdapWindow ();
     
     gconf_key = gconf_entry_get_key (e);
 
@@ -1062,7 +1169,7 @@ contacts_list_group_content_changed_nt (GConfClient *client, guint cid,
       group_split = g_strsplit (gconf_key, CONTACTS_GROUPS_KEY, 2);
 
       if (group_split [1])
-	group_name = g_strdup (group_split [1]);
+	group_name = g_utf8_strdown (group_split [1], -1);
 
       if (group_name) {
 
@@ -1070,23 +1177,25 @@ contacts_list_group_content_changed_nt (GConfClient *client, guint cid,
 		gtk_notebook_get_nth_page (GTK_NOTEBOOK (lw->notebook),
 					   cpt)) ){
 
-	  contact_section = 
-	    (gchar *) g_object_get_data (G_OBJECT (page), "contact_section");
+	  lwp = gnomemeeting_get_ldap_window_page (page);
 
-	  if (contact_section && !strcmp (contact_section, group_name)) 
+	  if (lwp && lwp->contact_section_name
+	      && !strcasecmp (lwp->contact_section_name, group_name)) 
 	    break;
 
 	  cpt++;
 	}
 
-	if (page) {
+	if (lwp) {
 
 	  list_store =
-	    GTK_LIST_STORE (g_object_get_data (G_OBJECT (page), "list_store"));
-
-	  gnomemeeting_addressbook_group_populate (list_store, group_name);
+	    GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (lwp->tree_view)));
+	  group_name_unescaped =
+	    gconf_unescape_key (group_name, -1);
+	  gnomemeeting_addressbook_group_populate (list_store,
+						   group_name_unescaped);
+	  g_free (group_name_unescaped);
 	}
-	
 	g_free (group_name);
       }
 
@@ -1099,114 +1208,18 @@ contacts_list_group_content_changed_nt (GConfClient *client, guint cid,
 
   
 /* DESCRIPTION  :  This callback is called when something changes in the 
- * 		   servers contacts list. 
+ * 		   servers or groups contacts list. 
  * BEHAVIOR     :  It updates the tree_view widget and the notebook pages.
- * PRE          :  /
+ * PRE          :  data is the page type (CONTACTS_SERVERS or CONTACTS_GROUPS)
  */
-static void contacts_list_changed_nt (GConfClient *client, guint cid,
-				      GConfEntry *e, gpointer data)
+static void contacts_sections_list_changed_nt (GConfClient *client, guint cid,
+					       GConfEntry *e, gpointer data)
 { 
-  GmLdapWindow *lw = NULL;
-
-  GtkWidget *page = NULL;
-
-  GtkTreeModel *model = NULL;
-  GtkTreeIter iter, child_iter;
-  
-  GSList *contacts_list = NULL;
-  GSList *contacts_list_iter = NULL;
-
-  gboolean non_empty = false;
-  gboolean found = false;
-  int cpt = 0;
-  int page_num = -1;
-  int page_type = 0;
-  const char *gconf_key = NULL;
-  gchar *path = NULL;
-  
-  
   if (e->value->type == GCONF_VALUE_LIST) {
   
     gdk_threads_enter ();
-    gconf_key = gconf_entry_get_key (e);
- 
-    lw = gnomemeeting_get_ldap_window (gm);
-    contacts_list = 
-      gconf_client_get_list (client, gconf_key, GCONF_VALUE_STRING, NULL);   
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW (lw->tree_view));
-    
-    /* Populate the GtkTreeStore and create the corresponding notebook 
-     * pages */
-    contacts_list_iter = contacts_list;
-    path = g_strdup_printf ("%d", GPOINTER_TO_INT (data));
-    gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (model), &iter, 
-					 path);
-    g_free (path);
-
-    non_empty = 
-      gtk_tree_model_iter_children (GTK_TREE_MODEL (model), 
-				    &child_iter, &iter);
-	
-    /* Remove all the servers */
-    while (non_empty && child_iter.stamp != 0) 
-      gtk_tree_store_remove (GTK_TREE_STORE (model), &child_iter);
-  
-    /* Clean the notebook from pages that disappeared from the list */
-    while ((page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (lw->notebook), 
-					      cpt))){
-      gchar *contact_section2 =  
-	(gchar *) g_object_get_data (G_OBJECT (page), "contact_section");
-
-      contacts_list_iter = contacts_list;
-      found = false;
-      while (contact_section2 && contacts_list_iter) {
-
-	if (!strcmp (contact_section2, (char *) contacts_list_iter->data)) {
-	 
-	  found = true;
-	  break;
-	}
-      	
-	contacts_list_iter = contacts_list_iter->next;
-      }
-
-      page_type =
-	GPOINTER_TO_INT (g_object_get_data (G_OBJECT (page), "page_type"));
-      if (!found && GPOINTER_TO_INT (data) == page_type) {
-
-	gtk_notebook_remove_page (GTK_NOTEBOOK (lw->notebook), cpt);
-	cpt = -1;
-      }	
-      
-      cpt++;
-    }
-
-    
-    contacts_list_iter = contacts_list;
-    /* Add all servers to the notebook if they are not already present */
-    while (contacts_list_iter) {
-
-      const char *contact_section = (const char *) contacts_list_iter->data;
- 
-      /* This will only add a page to the notebook if there was no page
-       * for the given server name */
-      page_num = 
-	gnomemeeting_init_ldap_window_notebook ((gchar *) contact_section,
-						GPOINTER_TO_INT (data));
-    
-      gtk_tree_store_append (GTK_TREE_STORE (model), &child_iter, &iter);
-      gtk_tree_store_set (GTK_TREE_STORE (model), &child_iter, 0, 
-			  contact_section, 1, page_num, -1);
-      contacts_list_iter = contacts_list_iter->next;
-
-      cpt++;
-    }
-
-    gtk_tree_view_expand_all (GTK_TREE_VIEW (lw->tree_view));
- 
+    gnomemeeting_addressbook_sections_populate ();
     gdk_threads_leave ();
-
-    g_slist_free (contacts_list);
   }
 }
 
@@ -1220,8 +1233,10 @@ static void contacts_list_changed_nt (GConfClient *client, guint cid,
 static void forward_toggle_changed_nt (GConfClient *client, guint cid, 
 				       GConfEntry *entry, gpointer data)
 {
+  /* FIX ME: could be moved */
   GmWindow *gw = NULL;
   gchar *gconf_string = NULL;
+  GMURL url;
   GtkWidget *msg_box = NULL;
 
 
@@ -1230,12 +1245,14 @@ static void forward_toggle_changed_nt (GConfClient *client, guint cid,
  
     gdk_threads_enter ();
 
-    gw = gnomemeeting_get_main_window (gm);
+    gw = MyApp->GetMainWindow ();
 
     /* Checks if the forward host name is ok */
     gconf_string =  gconf_client_get_string (GCONF_CLIENT (client), "/apps/gnomemeeting/call_forwarding/forward_host", NULL);
-      
-    if ((gconf_string == NULL) || (!strcmp (gconf_string, ""))) {
+
+    if (gconf_string)
+      url = GMURL (gconf_string);
+    if (url.IsEmpty ()) {
 	
       msg_box = 
 	gtk_message_dialog_new (GTK_WINDOW (gw->pref_window),
@@ -1324,12 +1341,12 @@ static void do_not_disturb_changed_nt (GConfClient *client, guint cid,
       ils_client->Modify ();
 
     gdk_threads_enter ();
-    gw = gnomemeeting_get_main_window (gm);
+    gw = MyApp->GetMainWindow ();
 
     if (gconf_value_get_bool (entry->value))
-      gnomemeeting_tray_set_content (G_OBJECT (gw->docklet), 2);
+      gnomemeeting_tray_set_content (gw->docklet, 2);
     else
-      gnomemeeting_tray_set_content (G_OBJECT (gw->docklet), 0);
+      gnomemeeting_tray_set_content (gw->docklet, 0);
     gdk_threads_leave ();
   }
 }
@@ -1358,14 +1375,16 @@ static void network_settings_changed_nt (GConfClient *client, guint,
 static void 
 lid_aec_changed_nt (GConfClient *client, guint, GConfEntry *entry, gpointer)
 {
+  GMH323EndPoint *ep = NULL;
+  GMLid *lid = NULL;
+  int lid_aec = 0;
+  
   if (entry->value->type == GCONF_VALUE_INT) {
 
-    int lid_aec = gconf_value_get_int (entry->value);
-    OpalLineInterfaceDevice *lid = NULL;
-    GMLid *lid_thread = MyApp->Endpoint ()->GetLidThread ();
+    lid_aec = gconf_value_get_int (entry->value);
 
-    if (lid_thread)
-      lid = lid_thread->GetLidDevice ();
+    ep = MyApp->Endpoint ();
+    lid = (ep ? ep->GetLid () : NULL);
 
     if (lid) {
 
@@ -1391,6 +1410,8 @@ lid_aec_changed_nt (GConfClient *client, guint, GConfEntry *entry, gpointer)
 	lid->SetAEC (0, OpalLineInterfaceDevice::AECAGC);
 	break;
       }
+
+      lid->Unlock ();
     }
   }
 }
@@ -1405,29 +1426,81 @@ static void
 lid_country_changed_nt (GConfClient *client, guint, GConfEntry *entry, 
 			gpointer)
 {
+  GMH323EndPoint *ep = NULL;
+  GMLid *lid = NULL;
+  gchar *country_code = NULL;
+  
   if (entry->value->type == GCONF_VALUE_STRING) {
     
-    GMLid *lid_thread = GM_LID (MyApp->Endpoint ()->GetLidThread ());
-    OpalLineInterfaceDevice *lid = NULL;
+    ep = MyApp->Endpoint ();
+    lid = (ep ? ep->GetLid () : NULL);
 
-    if (lid_thread)
-      lid = lid_thread->GetLidDevice ();
-
-    if ((gconf_value_get_string (entry->value))&&(lid))
-      lid->SetCountryCodeName (gconf_value_get_string (entry->value));
+    country_code = g_strdup (gconf_value_get_string (entry->value));
+    
+    if (country_code && lid) {
+      
+      lid->SetCountryCodeName (country_code);
+      lid->Unlock ();
+      g_free (country_code);
+    }
   }
 }
 #endif
 
 
-/* The functions  */
-void gnomemeeting_init_gconf (GConfClient *client)
+/* DESCRIPTION  :  This callback is called when a gconf error happens
+ * BEHAVIOR     :  Pop-up a message-box
+ * PRE          :  /
+ */
+static void
+gconf_error_callback (GConfClient *,
+		      GError *)
 {
-  GmDruidWindow *dw = gnomemeeting_get_druid_window (gm);
-  GmPrefWindow *pw = gnomemeeting_get_pref_window (gm);
-  GmWindow *gw = gnomemeeting_get_main_window (gm);
+  GtkWidget *dialog = 
+    gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
+			    GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+			    _("An error has happened in the configuration"
+			      " backend.\nMaybe some of your settings won't "
+			      "be saved."));
+
+  gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
+}
+
+
+/* The functions  */
+gboolean gnomemeeting_init_gconf (GConfClient *client)
+{
+  GmDruidWindow *dw = MyApp->GetDruidWindow ();
+  GmPrefWindow *pw = MyApp->GetPrefWindow ();
+  GmWindow *gw = MyApp->GetMainWindow ();
   MenuEntry *gnomemeeting_menu = gnomemeeting_get_menu (gm);
+#ifndef WIN32
   MenuEntry *tray_menu = gnomemeeting_get_tray_menu (gm);
+#endif
+  int gconf_test = -1;
+
+  
+#ifndef DISABLE_GCONF
+  gconf_client_add_dir (client, "/apps/gnomemeeting",
+			GCONF_CLIENT_PRELOAD_RECURSIVE, 0);
+#endif
+
+    
+#ifndef WIN32
+  gconf_test =
+    gconf_client_get_int (client, GENERAL_KEY "gconf_test_age", NULL);
+  
+  if (gconf_test != SCHEMA_AGE) 
+    return FALSE;
+#endif
+
+  
+  /* Set a default gconf error handler */
+  gconf_client_set_error_handling (gconf_client_get_default (),
+				   GCONF_CLIENT_HANDLE_UNRETURNED);
+  gconf_client_set_global_default_error_handler (gconf_error_callback);
+
 
   /* There are in general 2 notifiers to attach to each widget :
      - the notifier that will update the widget itself to the new key
@@ -1473,18 +1546,22 @@ void gnomemeeting_init_gconf (GConfClient *client)
   gconf_client_notify_add (client, "/apps/gnomemeeting/view/show_chat_window", view_widget_changed_nt, gw->chat_window, 0, 0);
 
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/auto_answer", menu_toggle_changed_nt, gnomemeeting_menu [AA_CALL_MENU_INDICE].widget, 0, 0);
+#ifndef WIN32
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/auto_answer", 
 			   menu_toggle_changed_nt,
 			   tray_menu [4].widget,
 			   0, 0);
+#endif
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/auto_answer", toggle_changed_nt, pw->aa, 0, 0);
 
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/do_not_disturb", toggle_changed_nt, pw->dnd, 0, 0);
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/do_not_disturb", menu_toggle_changed_nt, gnomemeeting_menu [DND_CALL_MENU_INDICE].widget, 0, 0);
+#ifndef WIN32
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/do_not_disturb",
 			   menu_toggle_changed_nt, 
 			   tray_menu [3].widget,
 			   0, 0);
+#endif
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/do_not_disturb", do_not_disturb_changed_nt, pw->dnd, 0, 0);
 
 
@@ -1494,6 +1571,7 @@ void gnomemeeting_init_gconf (GConfClient *client)
 #endif
 
   gconf_client_notify_add (client, "/apps/gnomemeeting/general/incoming_call_sound", toggle_changed_nt, pw->incoming_call_sound, 0, 0);
+  gconf_client_notify_add (client, GENERAL_KEY "auto_clear_text_chat", toggle_changed_nt, pw->auto_clear_text_chat, 0, 0);
 
 
   /* gnomemeeting_init_pref_window_h323_advanced */
@@ -1669,15 +1747,15 @@ void gnomemeeting_init_gconf (GConfClient *client)
 
   /* LDAP Window */
   gconf_client_notify_add (client, CONTACTS_KEY "ldap_servers_list",
-			   contacts_list_changed_nt, 
+			   contacts_sections_list_changed_nt, 
 			   GINT_TO_POINTER (CONTACTS_SERVERS), 0, 0);	    
 
   gconf_client_notify_add (client, CONTACTS_KEY "groups_list",
-			   contacts_list_changed_nt, 
+			   contacts_sections_list_changed_nt, 
 			   GINT_TO_POINTER (CONTACTS_GROUPS), 0, 0);	     
 
   gconf_client_notify_add (client, CONTACTS_KEY "groups",
-			   contacts_list_group_content_changed_nt, 
+			   contacts_sections_list_group_content_changed_nt, 
 			   NULL, 0, 0);
 
   
@@ -1686,9 +1764,174 @@ void gnomemeeting_init_gconf (GConfClient *client)
   gconf_client_notify_add (client, SERVICES_KEY "enable_microtelco",
 			   microtelco_enabled_nt, NULL, 0, 0);
 #endif
+
+  return TRUE;
 }
 
 
+void gnomemeeting_gconf_upgrade ()
+{
+  int gconf_value_int = 0;
+  gchar *gconf_value = NULL;
+  gchar *gconf_url = NULL;
+  gchar *group_name = NULL;
+  gchar *group_content_gconf_key = NULL;
+  gchar *new_group_content_gconf_key = NULL;
+  GSList *group_content = NULL;
+  GSList *group_content_iter = NULL;
+  GSList *new_group_content = NULL;
+  GSList *groups = NULL;
+  GSList *groups_iter = NULL;
+  GSList *list = NULL;
+
+  int version = 0;
+  GConfClient *client = NULL;
+
+  client = gconf_client_get_default ();
+  version = gconf_client_get_int (client, GENERAL_KEY "version", NULL);
+  
+  /* New Speex Audio codec in 0.95 (all Unix versions of 0.95 will have it)
+     Also enable Fast Start and enable Tunneling */
+  if (version < 95) {
+
+    list = g_slist_append (list, (void *) "SpeexNarrow-8k=1");
+    list = g_slist_append (list, (void *) "MS-GSM=1");
+    list = g_slist_append (list, (void *) "SpeexNarrow-15k=1");
+    list = g_slist_append (list, (void *) "GSM-06.10=1");
+    list = g_slist_append (list, (void *) "G.726-32k=1");
+    list = g_slist_append (list, (void *) "G.711-uLaw-64k=1");
+    list = g_slist_append (list, (void *) "G.711-ALaw-64k=1");
+    list = g_slist_append (list, (void *) "LPC-10=1");
+    list = g_slist_append (list, (void *) "G.723.1=1");
+    gconf_client_set_list (client, AUDIO_CODECS_KEY "codecs_list", 
+			   GCONF_VALUE_STRING, list, NULL);
+
+    g_slist_free (list);
+
+    gconf_client_set_bool (client, GENERAL_KEY "fast_start", false, NULL);
+    gconf_client_set_bool (client, GENERAL_KEY "h245_tunneling", true, NULL);
+  }
+
+
+  /* With 0.97, we convert the old addressbook to the new format,
+     same for the port ranges */
+  if (version < 97) {
+
+    groups =
+      gconf_client_get_list (client, CONTACTS_KEY "groups_list",
+			     GCONF_VALUE_STRING, NULL);
+    groups_iter = groups;
+  
+    while (groups_iter && groups_iter->data) {
+    
+      group_name = g_utf8_strdown ((char *) groups_iter->data, -1);
+      group_content_gconf_key =
+	g_strdup_printf ("%s%s", CONTACTS_GROUPS_KEY,
+			 (char *) groups_iter->data);
+      new_group_content_gconf_key =
+	g_strdup_printf ("%s%s", CONTACTS_GROUPS_KEY, group_name);
+	
+      group_content =
+	gconf_client_get_list (client, group_content_gconf_key,
+			       GCONF_VALUE_STRING, NULL);
+      group_content_iter = group_content;
+      
+      new_group_content =
+	gconf_client_get_list (client, new_group_content_gconf_key,
+			       GCONF_VALUE_STRING, NULL);
+	
+      while (group_content_iter && group_content_iter->data) {
+
+	new_group_content =
+	  g_slist_append (new_group_content, group_content_iter->data);
+	  
+	group_content_iter = g_slist_next (group_content_iter);
+      }
+
+      gconf_client_set_list (client, new_group_content_gconf_key,
+			     GCONF_VALUE_STRING, new_group_content, NULL);
+      gconf_client_remove_dir (client, "/apps/gnomemeeting", 0);
+      gconf_client_unset (client, group_content_gconf_key, NULL);
+      gconf_client_add_dir (client, "/apps/gnomemeeting",
+			    GCONF_CLIENT_PRELOAD_RECURSIVE, 0);
+      g_free (group_content_gconf_key);
+      g_free (new_group_content_gconf_key);
+      g_free (group_name);
+      g_slist_free (group_content);
+      g_slist_free (new_group_content);
+      new_group_content = NULL;
+      groups_iter = g_slist_next (groups_iter);
+    }
+      
+    g_slist_free (groups);
+
+
+    /* Convert the old ports keys */
+    gconf_value_int =
+      gconf_client_get_int (client, GENERAL_KEY "listen_port", 0);
+    if (gconf_value_int != 0)
+      gconf_client_set_int (client, PORTS_KEY "listen_port",
+			    gconf_value_int, 0);
+    
+    gconf_value =
+      gconf_client_get_string (client, GENERAL_KEY "tcp_port_range", 0);
+    if (gconf_value)
+      gconf_client_set_string (client, PORTS_KEY "tcp_port_range",
+			       gconf_value, 0);
+    g_free (gconf_value);
+    
+    gconf_value =
+      gconf_client_get_string (client, GENERAL_KEY "udp_port_range", 0);
+    if (gconf_value)
+      gconf_client_set_string (client, PORTS_KEY "rtp_port_range",
+			       gconf_value, 0);
+    g_free (gconf_value);
+
+    gconf_client_remove_dir (client, "/apps/gnomemeeting", 0);
+    gconf_client_unset (client, GENERAL_KEY "listen_port", NULL);
+    gconf_client_unset (client, GENERAL_KEY "tcp_port_range", NULL);
+    gconf_client_unset (client, GENERAL_KEY "udp_port_range", NULL);
+    gconf_client_add_dir (client, "/apps/gnomemeeting",
+			  GCONF_CLIENT_PRELOAD_RECURSIVE, 0);
+  }
+
+  
+  /* Install the URL Handlers */
+  gconf_url = 
+    gconf_client_get_string (client, 
+			     "/desktop/gnome/url-handlers/callto/command", 0);
+					       
+  if (!gconf_url) {
+    
+    gconf_client_set_string (client,
+			     "/desktop/gnome/url-handlers/callto/command", 
+			     "gnomemeeting -c \"%s\"", NULL);
+    gconf_client_set_bool (client,
+			   "/desktop/gnome/url-handlers/callto/need-terminal", 
+			   false, NULL);
+    gconf_client_set_bool (client,
+			   "/desktop/gnome/url-handlers/callto/enabled", 
+			   true, NULL);
+  }
+  g_free (gconf_url);
+
+  gconf_url = 
+    gconf_client_get_string (client, 
+			     "/desktop/gnome/url-handlers/h323/command", 0);
+  if (!gconf_url) {
+    
+    gconf_client_set_string (client,
+			     "/desktop/gnome/url-handlers/h323/command", 
+			     "gnomemeeting -c \"%s\"", NULL);
+    gconf_client_set_bool (client,
+			   "/desktop/gnome/url-handlers/h323/need-terminal", 
+			   false, NULL);
+    gconf_client_set_bool (client,
+			   "/desktop/gnome/url-handlers/h323/enabled", 
+			   true, NULL);
+  }
+  g_free (gconf_url);
+}
 
 
 void entry_changed (GtkEditable  *e, gpointer data)
