@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2001 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,15 +15,23 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         connection.h  -  description
  *                         ----------------------------
  *   begin                : Sat Dec 23 2001
- *   copyright            : (C) 2000-2001 by Damien Sandras
+ *   copyright            : (C) 2000-2003 by Damien Sandras
  *   description          : This file contains connection related functions.
- *   email                : dsandras@seconix.com
  *
  */
 
@@ -31,13 +39,8 @@
 #ifndef _CONNECTION_H_
 #define _CONNECTION_H_
 
-#include <ptlib.h>
-#include <h323.h>
-#include <string.h>
-#include "h323pdu.h"
-
-#include "endpoint.h"
 #include "common.h"
+#include "endpoint.h"
 
 
 /* GMH323Connection */
@@ -71,23 +74,6 @@ class GMH323Connection : public H323Connection
   virtual void OnClosedLogicalChannel (H323Channel &);
 
 
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  if int = 0, pauses or unpauses the transmitted
-   *                 audio channel (if any)
-   *                 if int = 1, pauses or unpauses the transmitted
-   *                 video channel (if any)
-   * PRE          :  /
-   */
-  void PauseChannel (int);
-
-
-  /* DESCRIPTION  :  /
-   * BEHAVIOR     :  Unpause video and audio channels
-   * PRE          :  /
-   */
-  void UnPauseChannels (void);
-
-
   /* DESCRIPTION  :  This callback is called to give the opportunity
    *                 to take an action on an incoming call
    * BEHAVIOR     :  Behavior is the following :
@@ -111,6 +97,20 @@ class GMH323Connection : public H323Connection
   virtual void OnUserInputString(const PString &);
 
 
+  /* DESCRIPTION  :  This is called when the current call is transferred but
+   *                 that it fails.
+   * BEHAVIOR     :  Displays an error message.
+   * PRE          :  /
+   */
+  virtual void HandleCallTransferFailure(const int);
+
+
+  /* DESCRIPTION  :  This is called when we received Q.931 Facility
+   * BEHAVIOR     :  Detect if it's an H.245 reverting message
+   * PRE          :  /
+   */
+  virtual BOOL OnReceivedFacility(const H323SignalPDU &);
+
   protected:
     GmWindow *gw;
     int opened_channels; 
