@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2002 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,15 +15,24 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
- *                         gdkvideoio.cxx  -  description
- *                        -------------------
+ *                         gdkvideoio.h  -  description
+ *                         ----------------------------
  *   begin                : Sat Feb 17 2001
- *   copyright            : (C) 2000-2001 by Damien Sandras
- *   description          : Class to permit to display in GDK Drawing Area
- *   email                : dsandras@seconix.com
+ *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   description          : Class to permit to display in GDK Drawing Area or
+ *                          SDL.
  *
  */
 
@@ -31,22 +40,18 @@
 #ifndef _GDKVIDEOIO_H_
 #define _GDKVIDEOIO_H_
 
-#include <ptlib.h>
-#include <h323.h>
-#include <videoio.h>
-#include <gtk/gtk.h>
-#include <pthread.h>
+#include "common.h"
 
 #ifdef HAS_SDL
 #include <SDL.h>
 #endif
 
-#include "common.h"
 
 
-class GDKVideoOutputDevice : public H323VideoDevice
+
+class GDKVideoOutputDevice : public PVideoOutputDeviceRGB
 {
-  PCLASSINFO(GDKVideoOutputDevice, H323VideoDevice);
+  PCLASSINFO(GDKVideoOutputDevice, PVideoOutputDeviceRGB);
 
 
   public:
@@ -78,22 +83,72 @@ class GDKVideoOutputDevice : public H323VideoDevice
   ~GDKVideoOutputDevice ();
 
 
-  /* Same as in H323VideoDevice.
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Open the device given the device name.
+   * PRE          :  Device name to open, immediately start device.
    */
-  virtual BOOL Redraw(const void * frame);
+  BOOL Open (const PString &, BOOL) { return TRUE; }
+
+  
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Return a list of all of the drivers available.
+   * PRE          :  /
+   */
+  PStringList GetDeviceNames() const;
 
 
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Get the maximum frame size in bytes.
+   * PRE          :  /
+   */
+  PINDEX GetMaxFrameBytes() { return 352 * 288 * 3 * 2; }
+
+  
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns TRUE if the output device is open.
+   * PRE          :  /
+   */
+  BOOL IsOpen ();
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  If data for the end frame is received, then we convert
+   *                 it from to RGB32 and we display it.
+   * PRE          :  x and y positions in the picture (>=0),
+   *                 width and height (>=0),
+   *                 the data, and a boolean indicating if it is the end
+   *                 frame or not.
+   */
+  BOOL SetFrameData (unsigned, unsigned, unsigned, unsigned,
+		     const BYTE *, BOOL);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Returns TRUE if the colour format is supported (ie RGB24).
+   * PRE          :  /
+   */
+  BOOL SetColourFormat (const PString &);
+
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Displays the current frame.
+   * PRE          :  /
+   */
+  BOOL EndFrame();
+  
  protected:
 
-  /* Same as in H323VideoDevice */
-  BOOL WriteLineSegment(int x, int y, unsigned len, const BYTE * data);
+
+  /* DESCRIPTION  :  /
+   * BEHAVIOR     :  Redraw the frame given as parameter.
+   * PRE          :  /
+   */
+  BOOL Redraw ();
+
 
   int device_id;      /* The current device : encoding or not */
-  int transmitted_frame_number;
-  int received_frame_number;
-
-  PBYTEArray buffer;  /* The RGB24 buffer; contains the images */
   int display_config; /* Current display : local or remote or both */
+
   PMutex redraw_mutex;
 
 #ifdef HAS_SDL
