@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2001 Damien Sandras
+ * Copyright (C) 2000-2003 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,15 +15,23 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         ils.cpp  -  description
  *                         -----------------------
  *   begin                : Sun Sep 23 2001
- *   copyright            : (C) 2000-2002 by Damien Sandras
- *   description          : The ldap thread.
- *   email                : dsandras@seconix.com
+ *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   description          : The ILS class thread.
  *
  */
 
@@ -31,25 +39,14 @@
 #include "../config.h"
 
 #include "ils.h"
-#include "ldap_window.h"
 #include "gnomemeeting.h"
-#include "common.h"
+#include "ldap_window.h"
 #include "misc.h"
 #include "stock-icons.h"
 #include "dialog.h"
-#include "misc.h"
-#include "xdap.h"
 
-#include <sys/time.h>
-#include <ptlib.h>
 #include <ldap.h>
 
-
-#ifndef DISABLE_GNOME
-#include <gnome.h>
-#endif
-
-
 #include "../pixmaps/inlines.h"
 
 
@@ -119,9 +116,7 @@ process (LDAP * ldap, xmlDocPtr xp, xmlNodePtr * curp)
   } else if (msgid > 0) {
     
     gnomemeeting_threads_enter ();
-    gnomemeeting_error_dialog (GTK_WINDOW (gm),
-			       _("Parse error: %d %s."),
-			       msgid, pferrtostring (msgid));
+    gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Parse error"), _("There was an error while processing the registering to the user directory: %d %s."), msgid, pferrtostring (msgid));
     gnomemeeting_threads_leave ();
   }
 }
@@ -131,8 +126,8 @@ process (LDAP * ldap, xmlDocPtr xp, xmlNodePtr * curp)
 GMILSClient::GMILSClient ()
   :PThread (1000, NoAutoDeleteThread)
 {
-  gw = gnomemeeting_get_main_window (gm);
-  lw = gnomemeeting_get_ldap_window (gm);
+  gw = MyApp->GetMainWindow ();
+  lw = MyApp->GetLdapWindow ();
 
   running = 1;
   has_to_register = 0;
@@ -142,10 +137,6 @@ GMILSClient::GMILSClient ()
 
   client = gconf_client_get_default ();
 
-
-  ldap_search_connection = NULL;
-  rc_search_connection = -1;
-
   Resume ();
 }
 
@@ -188,6 +179,7 @@ void GMILSClient::Main ()
 
     /* if there is more than 20 minutes that we are registered,
        we refresh the entry */
+    gnomemeeting_threads_enter ();
     if ((t.GetSeconds () > 1200) && 
 	(gconf_client_get_bool (GCONF_CLIENT (client), 
 				"/apps/gnomemeeting/ldap/register", 
@@ -197,16 +189,11 @@ void GMILSClient::Main ()
 	has_to_register = 1;
 	starttime = PTime ();
       }
+    gnomemeeting_threads_leave ();
 
     Current ()->Sleep (100);
   }
 
-  /* After the thread has stopped,
-     we unregister if we were registered */
-  if (registered == 1)
-    Register (0);
-
-
   quit_mutex.Signal ();
 }
 
@@ -237,15 +224,13 @@ BOOL GMILSClient::CheckFieldsConfig ()
 			   "/apps/gnomemeeting/ldap/register", 
 			   NULL);
 
-
   if (registering) {
 
     if ((firstname == NULL) || (!strcmp (firstname, ""))
-	|| (surname == NULL) || (!strcmp (surname, ""))
 	|| (mail == NULL) || (!strcmp (mail, ""))) {
       
       /* No need to display that for unregistering */
-      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Please provide your first name, last name, and e-mail in the Personal Data section in order to be able to register to the XDAP server."));
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Invalid parameters"), _("Please provide your first name and e-mail in the Personal Data section in order to be able to register to the user directory."));
       
       no_error = FALSE;
     }
@@ -268,16 +253,19 @@ BOOL GMILSClient::CheckServerConfig ()
   gchar *ldap_server = NULL;
 
 
+  gnomemeeting_threads_enter ();
   ldap_server =  
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/ldap/ldap_server", 
 			     NULL);
+  gnomemeeting_threads_leave ();
+
 
   /* We check that there is an ILS server specified */
   if ((ldap_server == NULL) || (!strcmp (ldap_server, ""))) {
 
     gnomemeeting_threads_enter ();
-    gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Operation impossible since there is no XDAP server specified."));
+    gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Invalid user directory"), _("Operation impossible since there is no user directory specified."));
     gnomemeeting_threads_leave ();
 
     return FALSE;
@@ -337,12 +325,12 @@ BOOL GMILSClient::Register (int reg)
 
   if (CheckServerConfig ()) {
 
+    gnomemeeting_threads_enter ();
     ldap_server =  
       gconf_client_get_string (GCONF_CLIENT (client),
 			       "/apps/gnomemeeting/ldap/ldap_server", 
 			       NULL);
 
-    gnomemeeting_threads_enter ();
     msg = g_strdup_printf (_("Contacting %s..."), ldap_server);
 
     if (reg != 2)
@@ -356,8 +344,7 @@ BOOL GMILSClient::Register (int reg)
 			  xdap_getentity, &oldgetent, 1))) {
       
       gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (gm),
-				 _("Failed to parse XML file."));
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Failed to parse XML file"), _("There was an error while parsing the XML file. Please make sure that it is correctly installed in your system."));
       gnomemeeting_threads_leave ();
 
       no_error = FALSE;
@@ -368,9 +355,7 @@ BOOL GMILSClient::Register (int reg)
 				&cred, &method))) {
 
       gnomemeeting_threads_enter ();      
-      gnomemeeting_error_dialog (GTK_WINDOW (gm),
-				 _("Bad ldap information from XML file: %s."),
-				 pferrtostring (rc));
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Bad information"), _("Bad ldap information from XML file: %s."), pferrtostring (rc));
       gnomemeeting_threads_leave ();
 
       no_error = FALSE;
@@ -379,9 +364,7 @@ BOOL GMILSClient::Register (int reg)
     else if (!(ldap = ldap_init (ldap_server, 389))) {
       
       gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (gm),
-				 _("Failed to open ldap server %s:%d."),
-				 ldap_server, "389");
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Cannot contact the user directory"), _("Failed to contact the user directory %s:%d."), ldap_server, "389");
       gnomemeeting_threads_leave ();
 
       no_error = FALSE;
@@ -391,8 +374,7 @@ BOOL GMILSClient::Register (int reg)
 	     != LDAP_OPT_SUCCESS) {
      
       gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (gm),
-				 _("Failed to set time limit on ldap operations."));
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Cannot contact the user directory"), _("Failed to set a time limit on operations."));
       gnomemeeting_threads_leave ();
 
       no_error = FALSE;  
@@ -401,9 +383,7 @@ BOOL GMILSClient::Register (int reg)
     else if ((rc = ldap_bind_s (ldap, who, cred, method))) {
       
       gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (gm),
-				 _("Failed to bind to ldap server: %s."),
-				 ldap_err2string (rc));
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Cannot contact the user directory"), _("Failed to bind to user directory: %s."), ldap_err2string (rc));
       gnomemeeting_threads_leave ();
 
       no_error = FALSE;
@@ -412,12 +392,13 @@ BOOL GMILSClient::Register (int reg)
     else {
 
       gnomemeeting_threads_enter ();
-      if (CheckFieldsConfig ()) {
+      bool ok = CheckFieldsConfig ();
+      gnomemeeting_threads_leave ();
+      
+      if (ok) {
 	while (current) {
 
-	  gnomemeeting_threads_leave (); /* It is a long operation */
 	  process (ldap, xp, &current);
-	  gnomemeeting_threads_enter ();
 	}
       }
       else
@@ -440,10 +421,13 @@ BOOL GMILSClient::Register (int reg)
 	     to the gateway IP of the translation */
 	  
 	  gchar *ip = NULL;
+	  gnomemeeting_threads_enter ();
 	  gchar *mail =  
 	    gconf_client_get_string (GCONF_CLIENT (client), 
 				     "/apps/gnomemeeting/personal_data/mail", 
 				     NULL);
+	  gnomemeeting_threads_leave ();
+
 	  if ((mail)&&(strcmp (mail, ""))
 	      &&(!strcmp (ldap_server, "ils.seconix.com")))
 	    ip = Search (ldap_server, "389", mail);
@@ -456,9 +440,11 @@ BOOL GMILSClient::Register (int reg)
 	    if (prt != P_MAX_INDEX)
 	      IP = IP.Left (prt);
 	    
-	    if (PIPSocket::Address (IP) != PIPSocket::Address ("0.0.0.0"))
+	    gnomemeeting_threads_enter ();
+	    if (PString ("0.0.0.0") != IP)
 	      gconf_client_set_string (client, "/apps/gnomemeeting/general/public_ip", (const char *) IP, NULL);
-	    
+	    gnomemeeting_threads_leave ();
+
 	    g_free (ip);
 	    g_free (mail);
 	  }
@@ -480,6 +466,8 @@ BOOL GMILSClient::Register (int reg)
 	registered = 0;
       }
 
+      gnomemeeting_threads_enter ();
+      
       if (reg != 2)
 	gnomemeeting_statusbar_flash (gw->statusbar, msg);
       gnomemeeting_log_insert (gw->history_text_view, msg);
@@ -493,6 +481,12 @@ BOOL GMILSClient::Register (int reg)
 
 
   xmlFreeDoc (xp);
+#if !HAVE_XMLREGISTERNODEDEFAULT
+  xdapfree();
+#endif
+#if XDAPLEAKCHECK
+  xdapleakcheck();
+#endif
 
   if (!no_error) {
 
@@ -506,7 +500,6 @@ BOOL GMILSClient::Register (int reg)
     
     gnomemeeting_log_insert (gw->history_text_view, msg);
     gnomemeeting_statusbar_flash (gw->statusbar, msg);
-    gnomemeeting_error_dialog (GTK_WINDOW (gm), msg);
     g_free (msg);
 
     has_to_register = 0;
@@ -524,6 +517,9 @@ BOOL GMILSClient::Register (int reg)
 
 gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
 {
+  LDAP *ldap_search_connection = NULL;
+  int rc_search_connection = -1;
+
   char *attrs [] = { "rfc822mailbox", "sappid", "sipaddress", "sport", NULL };
   char **ldv = NULL;
  
@@ -534,6 +530,7 @@ gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
   int part4;
   int port = 1720;
   struct timeval t = {10, 0};
+  struct timeval t2 = {7, 0};
 
   bool no_error = TRUE;
   gchar *ip = NULL;
@@ -542,20 +539,12 @@ gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
 
   LDAPMessage *res = NULL, *e = NULL;
 
-
   if (((!strcmp (ldap_server, ""))&&(ldap_server)) 
       ||((!strcmp (ldap_port, ""))&&(ldap_port)) 
       ||((!strcmp (mail, ""))&&(mail)))
     return NULL;
  
 
-  if ((ldap_search_connection != NULL)||(rc_search_connection != -1)) {
-    
-    ldap_abandon (ldap_search_connection, rc_search_connection);
-	
-  }
-
-
   /* must be able to reach ldap server */
   if (!(ldap_search_connection = ldap_init (ldap_server, atoi (ldap_port)))) {
       
@@ -582,7 +571,7 @@ gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
     rc_search_connection = 
       ldap_search_st (ldap_search_connection, "objectClass=RTPerson", 
 		      LDAP_SCOPE_BASE,
-		      cn, attrs, 0, &t, &res); 
+		      cn, attrs, 0, &t2, &res); 
     g_free (cn);
     
     if (rc_search_connection != 0)
@@ -625,16 +614,14 @@ gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
       }
 
     }
-    
-    part1 = (int) (nmip/(256*256*256));
-    part2 = (int) ((nmip - part1 * (256 * 256 * 256)) / (256 * 256));
-    part3 = (int) ((nmip - part1 * (256 * 256 * 256) - part2 * (256 * 256)) 
-		   / 256);
-    part4 = (int) ((nmip - part1 * (256 * 256 * 256) - part2 * (256 * 256) 
-		    - part3 * 256));
+
+    part1 = (nmip & 0xff000000) >> 24;
+    part2 = (nmip & 0x00ff0000) >> 16;
+    part3 = (nmip & 0x0000ff00) >> 8;
+    part4 = nmip & 0x000000ff;
     
     ip = g_strdup_printf ("%d.%d.%d.%d:%d", part4, part3, part2, part1, port);
-    
+
     ldap_msgfree (res);
     ldap_unbind (ldap_search_connection);
   }
@@ -642,7 +629,7 @@ gchar *GMILSClient::Search (gchar *ldap_server, gchar *ldap_port, gchar *mail)
 
   rc_search_connection = -1;
   ldap_search_connection = NULL;
-
+  
   return ip;
 }
 
@@ -659,6 +646,7 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
   gchar *mail = NULL;
   gchar *comment = NULL;
   gchar *location = NULL;
+  gchar *tmp = NULL;
   gchar *version = NULL;
   gchar *busy = NULL;
   gchar *ip = NULL;
@@ -668,6 +656,7 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
   
   unsigned long int sip = 0;
 
+  gnomemeeting_threads_enter ();
   GConfClient *client = gconf_client_get_default ();
 
 
@@ -680,30 +669,44 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/personal_data/lastname", 
 			     NULL);
-
+  if (!surname || !strcmp (surname, ""))
+    surname = g_strdup ("-");
+  tmp = g_strdup_printf ("%.65s", surname);
+  g_free (surname);
+  surname = tmp;
+  
   mail =  
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/personal_data/mail", 
 			     NULL);
+  tmp = g_strdup_printf ("%.65s", mail);
+  g_free (mail);
+  mail = tmp;
 
   comment =  
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/personal_data/comment", 
 			     NULL);
   if (!comment || !strcmp (comment, ""))
-    comment = g_strdup (" ");
+    comment = g_strdup ("-");
+  tmp = g_strdup_printf ("%.65s", comment);
+  g_free (comment);
+  comment = tmp;
 
   location =  
     gconf_client_get_string (GCONF_CLIENT (client),
 			     "/apps/gnomemeeting/personal_data/location", 
 			     NULL);
   if (!location || !strcmp (location, ""))
-    location = g_strdup (" ");
+    location = g_strdup ("-");
+  tmp = g_strdup_printf ("%.65s", location);
+  g_free (location);
+  location = tmp;
 
   port = 
     g_strdup_printf ("%d", 
 		     gconf_client_get_int (GCONF_CLIENT (client),
-					   "/apps/gnomemeeting/general/listen_port",
+					   PORTS_KEY "listen_port",
 					   NULL));
 
   version =  g_strdup_printf ("%u", MAJOR_VERSION << 24 | 
@@ -712,7 +715,7 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
 
   if ((MyApp->Endpoint ()->GetCallingState () != 0)
       || (gconf_client_get_bool (client, 
-				 "/apps/gnomemeeting/general/do_not_disturb", 
+				 GENERAL_KEY "do_not_disturb", 
 				 NULL)))
     busy = g_strdup ("1");
   else
@@ -727,6 +730,7 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
     ilsa32964638 = g_strdup ("1");
   else
     ilsa32964638 = g_strdup ("0");
+  gnomemeeting_threads_leave ();
 
   ip = MyApp->Endpoint ()->GetCurrentIP ();
   sip = inet_addr (ip);
@@ -812,10 +816,13 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
 
 /* The browser methods */
 
-GMILSBrowser::GMILSBrowser (gchar *server, gchar *filter)
+GMILSBrowser::GMILSBrowser (GmLdapWindowPage *lwpage,
+			    gchar *server,
+			    gchar *filter)
   :PThread (1000, AutoDeleteThread)
 {
-  lw = gnomemeeting_get_ldap_window (gm);
+  lw = MyApp->GetLdapWindow ();
+  lwp = lwpage;
   
   if (server)
     ldap_server = g_strdup (server);
@@ -823,13 +830,12 @@ GMILSBrowser::GMILSBrowser (gchar *server, gchar *filter)
     ldap_server = NULL;
 
   if (filter)
-    search_filter = g_convert (filter, strlen (filter), "ISO-8859-1", "UTF-8",
+    search_filter = g_convert (filter, strlen (filter),
+			       "ISO-8859-1", "UTF-8",
 			       NULL, NULL, NULL);
   else
     search_filter = NULL;
 
-  page = NULL;
-
   this->Resume ();
 }
 
@@ -842,52 +848,52 @@ GMILSBrowser::~GMILSBrowser ()
   if (search_filter)
     g_free (search_filter);
 
-  /* Removes the data associated with the page */
-  if (page)
-    g_object_set_data (G_OBJECT (page), "GMILSBrowser", NULL);
+  if (lwp) {
+
+    lwp->ils_browser = NULL;
+    lwp->search_quit_mutex.Signal ();
+  }
 }
 
 
 void GMILSBrowser::Main ()
 {
-  char *datas [] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
   char *attrs [] = { "surname", "givenname", "comment", "location", 
 		     "rfc822mailbox", "sipaddress", "ilsa32833566", 
 		     "ilsa32964638", "ilsa26279966", "ilsa26214430", 
-		     "sport", "sappid", "xstatus", 
-		     NULL };
+		     "sport", "sappid", "xstatus", NULL};
 
-  char **ldv;
+  char **char_ldap_data [] = {NULL, NULL, NULL, NULL, NULL, NULL};
+  char **bool_ldap_data [] = {NULL, NULL, NULL};
+  bool b_ldap_data [] = {false, false, false};
+  char * utf8_char_ldap_data [] = {NULL, NULL, NULL, NULL, NULL, NULL};
 
+  gchar **num_users = NULL;
+  char **sipaddress = NULL;
+  char **ilsa26279966 = NULL;
+  char **sport = NULL;
+  char **xstatus = NULL;
+  gchar *utf8_username = NULL;
+  gchar *utf8_remote_app = NULL;
+  gchar *utf8_callto = NULL;
+    
+  int rc = -1;
   LDAPMessage *res = NULL, *e = NULL;
 
+  unsigned int v = 0, a = 0, b = 0, c = 0;
+  unsigned int part1 = 0, part2 = 0, part3 = 0, part4 = 0;
   unsigned long int nmip = 0;
-  int part1;
-  int part2;
-  int part3;
-  int part4;
   int port = 1720;
-  int cj = 0;
   int users_nbr = 0;
-  int page_exists = 0; /* flag to see if the page still exists 
-			  when filling the list */
-  bool audio = false;
-  bool video = false; 
-  bool available = true;
+  int retry = 0;
 
-  char ip [16];
-  gchar *text_label = NULL;
-  gchar *msg = NULL;
   gchar *color = NULL;
   gchar *filter = NULL;
-  gchar *name = NULL;
-  gchar *callto = NULL;
-  gchar **num_users = NULL;
-  
-  GtkWidget *statusbar = NULL;
-  GtkListStore *xdap_users_list_store = NULL;
-  GtkWidget *xdap_users_tree_view = NULL;
+  gchar *ip = NULL;
+
+  LDAP *ldap_connection = NULL;
   GdkPixbuf *status_icon = NULL;
+  GtkListStore *users_list_store = NULL;
   GtkTreeIter list_iter;
 
   GmWindow *gw = NULL;
@@ -896,372 +902,324 @@ void GMILSBrowser::Main ()
   time_limit.tv_sec = 10;
   time_limit.tv_usec = 0;
   bool no_error = TRUE;
-  
-  msg = g_strdup_printf (_("Contacting %s..."), ldap_server);
 
-  gnomemeeting_threads_enter ();
-  /* Check if the page with the ldap_server we are browsing still exists */
-  while ((page = gtk_notebook_get_nth_page (GTK_NOTEBOOK (lw->notebook), 
-					    cj))) {
-    
-    text_label = 
-      (gchar *) g_object_get_data (G_OBJECT (page), "contact_section");
-    if (!(strcasecmp (text_label, ldap_server))) {
-      
-      page_exists = 1;
-      break;
-    }
-    
-    cj++;
-  }
-    
-  /* If we found a page, we have the LIST_STORE, if not, we can clean
-     the statusbar */
-  if (page != NULL) {
-    
-    xdap_users_list_store = 
-      GTK_LIST_STORE (g_object_get_data (G_OBJECT (page), "list_store"));
-    xdap_users_tree_view =
-      GTK_WIDGET (g_object_get_data (G_OBJECT (page), "tree_view"));
-    statusbar = 
-      GTK_WIDGET (g_object_get_data (G_OBJECT (page), "statusbar"));
-  }
-  else 
+  if (!lwp)
     return;
 
-  gw = gnomemeeting_get_main_window (gm);
-  gnomemeeting_statusbar_push (statusbar, msg);
-  g_free (msg);
+  lwp->search_quit_mutex.Wait ();
+
+  gnomemeeting_threads_enter ();
+  gw = MyApp->GetMainWindow ();  
+  users_list_store =
+    GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (lwp->tree_view)));
   gnomemeeting_threads_leave ();
 
-  /* must be able to reach ldap server */
-  if (!(ldap_connection = ldap_init (ldap_server, 389))) {
-      
+  do {
+
     gnomemeeting_threads_enter ();
-    gnomemeeting_error_dialog (GTK_WINDOW (gw->ldap_window),
-			       _("Failed to open ldap server %s:%d."),
-			       ldap_server, "389");
+    gnomemeeting_statusbar_push (lwp->statusbar,
+				 _("Contacting %s..."), ldap_server);
     gnomemeeting_threads_leave ();
 
-    no_error = FALSE;
-  }
-  /* Timeout */
-  else if (ldap_set_option (ldap_connection, LDAP_OPT_NETWORK_TIMEOUT, 
-			    &time_limit)
-	   != LDAP_OPT_SUCCESS) {
-     
-    gnomemeeting_threads_enter ();
-    gnomemeeting_error_dialog (GTK_WINDOW (gw->ldap_window),
-			       _("Failed to set time limit on ldap operations."));
-    gnomemeeting_threads_leave ();
-    
-    no_error = FALSE;  
-  }
-  /* must be able to bind to ldap server */
-  else if ((rc = ldap_bind_s (ldap_connection, NULL, NULL, 
-			      LDAP_AUTH_SIMPLE))) {
-    
-    gnomemeeting_threads_enter ();
-    gnomemeeting_error_dialog (GTK_WINDOW (gw->ldap_window),
-			       _("Failed to bind to ldap server %s: %s."),
-			       ldap_server, ldap_err2string (rc));
-    gnomemeeting_threads_leave ();
-    
-    no_error = FALSE;
-  }
-  /* all successful */
-  else {
-    
-    gnomemeeting_threads_enter ();        
-    msg = g_strdup_printf (_("Fetching user information from %s."),
-			   ldap_server);
-    gnomemeeting_statusbar_push (statusbar, msg);
-    gnomemeeting_threads_leave ();
-    g_free (msg);
-
-    if (search_filter)
-      filter = g_strdup_printf ("(&(cn=%%)%s)", search_filter);
-    else
-      filter = g_strdup ("(&(cn=%))");
-
-    
-    rc = ldap_search_s (ldap_connection, "objectClass=RTPerson", 
-			LDAP_SCOPE_BASE,
-			filter,	attrs, 0, &res); 
-    g_free (filter);
-
-    for (int i = 0 ; i < 9 ; i++)
-      datas [i] = NULL;
-    
-    
-    /* Maybe the user closed the tab while we were waiting */
-    if ((xdap_users_list_store != NULL) && (page_exists)) { 
+    /* must be able to reach ldap server */
+    if (!(ldap_connection = ldap_init (ldap_server, 389))) {
       
-      for (e = ldap_first_entry(ldap_connection, res); 
-	   e != NULL; e = ldap_next_entry(ldap_connection, e)) {
-	
-	users_nbr++;
-
-	ldv = ldap_get_values (ldap_connection, e, "surname");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	
-	  datas [3] = 
-	    g_strdup (ldv [0]);
-	  ldap_value_free (ldv);
-	}      
-	
-	ldv = ldap_get_values(ldap_connection, e, "givenname");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	  
-	  datas [2] = g_strdup (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-	
-	ldv = ldap_get_values(ldap_connection, e, "location");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	  
-	  datas [5] = g_strdup (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-	
-	ldv = ldap_get_values(ldap_connection, e, "comment");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	  
-	  datas [6] = 
-	    g_strdup (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-
-	ldv = ldap_get_values(ldap_connection, e, "sappid");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	  
-	  datas [7]  =
-	    g_strdup (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-
-
-	ldv = ldap_get_values(ldap_connection, e, "sport");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-
-	  port = atoi (ldv [0]);	
-
-	  /* Ugly hack because Netmeeting sucks, it registers
-	     random ports to ILS, but it only supports 1720 */
-	  if (datas [7]&&!strcmp (datas [7], "ms-netmeeting")&&port == 1503)
-	    port = 1720;
-
-	  ldap_value_free (ldv);
-	}
-
-
-	/* Sent one time only by ils.seconix.com only */
-	ldv = ldap_get_values(ldap_connection, e, "xstatus");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-
-	  num_users = g_strsplit (ldv [0], ",", 0);
-
-	  ldap_value_free (ldv);
-	}
-
-	
-	ldv = ldap_get_values(ldap_connection, e, "ilsa26279966");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-
-	  gchar *value =
-	    g_strdup (ldv [0]);
-	
-	  unsigned int v, a, b, c;
-	  v = atoi (value);
-	  a = (v & 0xff000000) >> 24;
-	  b = (v & 0x00ff0000) >> 16;
-	  c = v & 0x0000ffff;
-	  g_free (value);
-	  
-	  if (datas [7] != NULL)
-	    datas [7] = g_strdup_printf ("%s %d.%d.%d", datas [7], a, b, c); 
-	  else
-	    datas [7] = g_strdup_printf ("%d.%d.%d", a, b, c); 
-	  
-	  ldap_value_free (ldv);
-	}
-	
-	ldv = ldap_get_values(ldap_connection, e, "rfc822mailbox");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	
-	  datas [4] = g_strdup (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-
-	
-	ldv = ldap_get_values(ldap_connection, e, "sipaddress");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	
-	  nmip = strtoul (ldv [0], NULL, 10);
-	  ldap_value_free (ldv);
-	}
-      
-	part1 = (int) (nmip/(256*256*256));
-	part2 = (int) ((nmip - part1 * (256 * 256 * 256)) / (256 * 256));
-	part3 = (int) ((nmip - part1 * (256 * 256 * 256) - part2 * (256 * 256)) 
-		       / 256);
-	part4 = (int) ((nmip - part1 * (256 * 256 * 256) - part2 * (256 * 256) 
-			- part3 * 256));
-	
-	sprintf (ip, "%d.%d.%d.%d:%d", part4, part3, part2, part1, port);
-	/* ip will be freed (char ip [16]), so we make a copy in datas [7] */
-	
-	datas [8] = g_strdup ((char *) ip);
-	
-	
-	/* Video Capable ? */
-	ldv = ldap_get_values(ldap_connection, e, "ilsa32964638");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	  
-	  video = 
-	    (bool)atoi (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-      
-
-	/* Audio Capable ? */
-	ldv = ldap_get_values(ldap_connection, e, "ilsa32833566");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	  
-	  audio = 
-	    (bool)atoi (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-      
-
-	/* Busy ? */
-	ldv = ldap_get_values(ldap_connection, e, "ilsa26214430");
-	if ((ldv != NULL)&&(ldv [0] != NULL)) {
-	  
-	  available = 
-	    !(bool) atoi (ldv [0]);
-	  ldap_value_free (ldv);
-	}
-	
-
-	gnomemeeting_threads_enter ();
-	if (available) {
-
-	  status_icon = gtk_widget_render_icon 
-	    (xdap_users_tree_view, 
-	     GM_STOCK_STATUS_AVAILABLE,
-	     GTK_ICON_SIZE_MENU, NULL);
-	  color = g_strdup ("black");
-	} 
-        else {
-	 
-	  status_icon = gtk_widget_render_icon 
-	    (xdap_users_tree_view, 
-	     GM_STOCK_STATUS_OCCUPIED,
-	     GTK_ICON_SIZE_MENU, NULL);
-	  color = g_strdup ("darkred");
-        }
-        
-	gnomemeeting_threads_leave ();
-
-				
-	/* Check if the window is still present or not */
-	if (lw) {
-	  
-	  gchar *utf8_data [7] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL};
-
-	  
-	  for (int j = 0 ; j < 7 ; j++) {
-	    
-	    /* Strings are already UTF-8-encoded for LDAP v3,
-	       but Microsoft ILS doesn't take care of that */
-	    if (datas [j + 2]) {
-	      
-	      /* If valid UTF-8, ok */
-	      if (g_utf8_validate (datas [j + 2], -1, NULL))
-		utf8_data [j] = g_strdup (datas [j+2]);
-	      else
-		utf8_data [j] = 
-		  gnomemeeting_from_iso88591_to_utf8 (PString (datas [j+2]));
-	    }
-	  }
-
-	  /* The user name and callto */
-	  name = g_strdup_printf ("%s %s", utf8_data [0], utf8_data [1]);
-	  callto = g_strdup_printf ("callto://%s/%s",
-				    ldap_server, utf8_data [2]);
-
-	  gnomemeeting_threads_enter ();
-
-	  gtk_list_store_append (xdap_users_list_store, &list_iter);
-
-	  gtk_list_store_set (xdap_users_list_store, &list_iter,
-			      COLUMN_ILS_STATUS, status_icon,
-			      COLUMN_ILS_AUDIO, audio,
-			      COLUMN_ILS_VIDEO, video,
-			      COLUMN_ILS_NAME, name,
-			      COLUMN_ILS_CALLTO, callto,
-			      COLUMN_ILS_LOCATION, utf8_data [3],
-			      COLUMN_ILS_COMMENT, utf8_data [4],
-			      COLUMN_ILS_VERSION, utf8_data [5],
-			      COLUMN_ILS_IP, utf8_data [6],
-			      COLUMN_ILS_COLOR, color,
-			      -1);
-          
-	  gnomemeeting_threads_leave ();
-	  
-	  for (int j = 0 ; j < 7 ; j++)
-	    if (utf8_data [j])
-	      g_free (utf8_data [j]);
-	  
-	}
-
-	g_object_unref (status_icon);
-	g_free (color);
-	g_free (name);
-	g_free (callto);
-
-	for (int j = 2 ; j <= 8 ; j++) {
-	  
-	  if (datas [j] != NULL)
-	    g_free (datas [j]);
-	  
-	  datas [j] = NULL;
-	}
-
-      } /* end of for */
-
-
       gnomemeeting_threads_enter ();
-      if (num_users && num_users [1]) {
-	
-	msg =
-	  g_strdup_printf (_("Search completed: %d user(s) listed on a total of %d user(s) from %s."),
-			   users_nbr, PMAX (atoi (num_users [1]), users_nbr), ldap_server);
-	g_strfreev (num_users);
-      }
-      else
-	msg = g_strdup_printf (_("Search completed: %d user(s) found on %s."),
-			       users_nbr, ldap_server);
-	
-      gnomemeeting_statusbar_push (statusbar, msg); 
-      g_free (msg);
+      gnomemeeting_statusbar_push (lwp->statusbar,
+				   _("Failed to contact LDAP server %s:%d."),
+				   ldap_server, "389");
       gnomemeeting_threads_leave ();
       
+      no_error = FALSE;
     }
+    /* Timeout */
+    else if (ldap_set_option (ldap_connection, LDAP_OPT_NETWORK_TIMEOUT, 
+			      &time_limit)
+	     != LDAP_OPT_SUCCESS) {
+     
+      gnomemeeting_threads_enter ();
+      gnomemeeting_statusbar_push (lwp->statusbar,
+				   _("Failed to set time limit on LDAP operations."));
+      gnomemeeting_threads_leave ();
+    
+      no_error = FALSE;  
+    }
+    /* must be able to bind to ldap server */
+    else if ((rc = ldap_bind_s (ldap_connection, NULL, NULL, 
+				LDAP_AUTH_SIMPLE))) {
+      
+      gnomemeeting_threads_enter ();
+      gnomemeeting_statusbar_push (lwp->statusbar, _("Failed to contact to LDAP server %s: %s."), ldap_server, ldap_err2string (rc));
+      gnomemeeting_threads_leave ();
+    
+      no_error = FALSE;
+    }
+    /* all successful */
+    else {
+    
+      gnomemeeting_threads_enter ();        
+      gnomemeeting_statusbar_push (lwp->statusbar,
+				   _("Fetching online users list from %s."),
+				   ldap_server);
+      gnomemeeting_threads_leave ();
+      
+      if (search_filter)
+	filter = g_strdup_printf ("(&(cn=%%)%s)", search_filter);
+      else
+	filter = g_strdup ("(&(cn=%))");
+
+      rc =
+	ldap_search_s (ldap_connection, "objectClass=RTPerson", 
+		       LDAP_SCOPE_BASE,
+		       filter, attrs, 0, &res); 
+      g_free (filter);
+
+      gnomemeeting_threads_enter ();
+      if (rc != 0)
+	if (rc == LDAP_SERVER_DOWN) 
+	  gnomemeeting_statusbar_push (lwp->statusbar,
+				       _("Connection to %s lost..."),
+				       ldap_server);
+	else
+	  gnomemeeting_statusbar_push (lwp->statusbar, _("Could not fetch online users list."));
+      gnomemeeting_threads_leave ();
+  
+      if (rc == LDAP_SERVER_DOWN)
+	PThread::Current ()->Sleep (1000);
+
+      retry++;
+    }
+  } while ((rc == LDAP_SERVER_DOWN) && (retry <= 5));
+
+
+  if (rc == 0 && users_list_store != NULL && res) { 
+
+    gnomemeeting_threads_enter ();
+    gtk_tree_view_set_model (GTK_TREE_VIEW (lwp->tree_view), NULL);
+    gtk_list_store_clear (GTK_LIST_STORE (users_list_store));
+    gnomemeeting_threads_leave ();
+	
+    for (e = ldap_first_entry (ldap_connection, res); 
+	 e != NULL;
+	 e = ldap_next_entry (ldap_connection, e)) {
+	
+      users_nbr++;
+
+      char_ldap_data [0] = ldap_get_values(ldap_connection, e, "givenname");
+      char_ldap_data [1] = ldap_get_values (ldap_connection, e, "surname");
+      char_ldap_data [2] = ldap_get_values(ldap_connection, e, "location");
+      char_ldap_data [3] = ldap_get_values(ldap_connection, e, "comment");
+      char_ldap_data [4] = ldap_get_values(ldap_connection, e, "sappid");
+      char_ldap_data [5] =
+	ldap_get_values(ldap_connection, e, "rfc822mailbox");
+      bool_ldap_data [0] =
+	ldap_get_values(ldap_connection, e, "ilsa32964638"); /* video */
+      bool_ldap_data [1] =
+	ldap_get_values(ldap_connection, e, "ilsa32833566"); /* audio */
+      bool_ldap_data [2] =
+	ldap_get_values(ldap_connection, e, "ilsa26214430"); /* available */
+
+      sport = ldap_get_values(ldap_connection, e, "sport");
+      xstatus = ldap_get_values(ldap_connection, e, "xstatus");
+      ilsa26279966 = ldap_get_values(ldap_connection, e, "ilsa26279966");
+      sipaddress = ldap_get_values(ldap_connection, e, "sipaddress");
+
+
+      /* Free it here because we need to keep the last value to use it
+	 after the for */
+      if (num_users)
+	g_strfreev (num_users);
+	
+
+      /* char LDAP data, to be converted to UTF-8 if not valid */
+      for (int i = 0 ; i < 6 ; i++) {
+	  
+	if (char_ldap_data [i] && char_ldap_data [i] [0]) {
+	  
+	  if (!g_utf8_validate (char_ldap_data [i] [0], -1, NULL))
+	    utf8_char_ldap_data [i] =
+	      gnomemeeting_from_iso88591_to_utf8 (PString (char_ldap_data [i] [0]));
+	  else
+	    utf8_char_ldap_data [i] =
+	      g_strdup (char_ldap_data [i] [0]);
+	    
+	  ldap_value_free (char_ldap_data [i]);
+	  char_ldap_data [i] = NULL;
+	}
+      }
+
+
+      /* bool LDAP data */
+      for (int i = 0 ; i < 3 ; i++) {
+
+	if (bool_ldap_data [i] && bool_ldap_data [i] [0]) {
+	    
+	  b_ldap_data [i] = (bool) atoi (bool_ldap_data [i] [0]);
+	  ldap_value_free (bool_ldap_data [i]);
+	  bool_ldap_data [i] = NULL;
+	}
+      }
+
+
+      /* Number of users: seconix.com specific */
+      if (xstatus && xstatus [0]) {
+
+	num_users = g_strsplit (xstatus [0], ",", 0);
+	ldap_value_free (xstatus);
+	xstatus = NULL;
+      }
+
+
+      /* Remote application port */
+      if (sport && sport [0]) {
+
+	port = atoi (sport [0]);
+
+	/* Ignore data port */
+	if (port == 1503 && sport [1])
+	  port = atoi (sport [1]);
+	  
+	ldap_value_free (sport);
+	sport = NULL;
+      }
+
+
+      /* Remote IP */
+      if (sipaddress && sipaddress [0]) {
+
+	nmip = strtoul (sipaddress [0], NULL, 10);
+	part1 = (nmip & 0xff000000) >> 24;
+	part2 = (nmip & 0x00ff0000) >> 16;
+	part3 = (nmip & 0x0000ff00) >> 8;
+	part4 = nmip & 0x000000ff;
+
+	ip =
+	  g_strdup_printf ("%d.%d.%d.%d:%d",
+			   part4, part3, part2, part1, port);
+
+	ldap_value_free (sipaddress);
+	sipaddress = NULL;
+      }
+
+
+      /* Remote application name and version */
+      if (ilsa26279966 && ilsa26279966 [0]) {
+	  
+	v = atoi (ilsa26279966 [0]);
+	a = (v & 0xff000000) >> 24;
+	b = (v & 0x00ff0000) >> 16;
+	c = v & 0x0000ffff;
+
+	utf8_remote_app =
+	  g_strdup_printf ("%s %d.%d.%d",
+			   utf8_char_ldap_data [4] ?
+			   utf8_char_ldap_data [4] : "",
+			   a, b, c); 
+
+	ldap_value_free (ilsa26279966);
+	ilsa26279966 = NULL;
+      }
+	
+
+      /* Status icon */
+      gnomemeeting_threads_enter ();
+      if (!b_ldap_data [2]) {
+
+	status_icon =
+	  gtk_widget_render_icon (lwp->tree_view, 
+				  GM_STOCK_STATUS_AVAILABLE,
+				  GTK_ICON_SIZE_MENU, NULL);
+	color = g_strdup ("black");
+      } 
+      else {
+	 
+	status_icon =
+	  gtk_widget_render_icon (lwp->tree_view, 
+				  GM_STOCK_STATUS_OCCUPIED,
+				  GTK_ICON_SIZE_MENU, NULL);
+	color = g_strdup ("#8a8a8a");
+      }        
+      gnomemeeting_threads_leave ();
+
+      utf8_username =
+	g_strdup_printf ("%s %s",
+			 utf8_char_ldap_data [0] ?
+			 utf8_char_ldap_data [0] : "",
+			 utf8_char_ldap_data [1] ?
+			 utf8_char_ldap_data [1] : "");
+      utf8_callto =
+	g_strdup_printf ("callto:%s/%s",
+			 ldap_server,
+			 utf8_char_ldap_data [5] ?
+			 utf8_char_ldap_data [5] : "");
+
+      gnomemeeting_threads_enter ();
+      gtk_list_store_append (users_list_store, &list_iter);
+      gtk_list_store_set (users_list_store, &list_iter,
+			  COLUMN_ILS_STATUS, status_icon,
+			  COLUMN_ILS_AUDIO, b_ldap_data [1],
+			  COLUMN_ILS_VIDEO, b_ldap_data [0],
+			  COLUMN_ILS_NAME, utf8_username,
+			  COLUMN_ILS_URL, utf8_callto,
+			  COLUMN_ILS_LOCATION, utf8_char_ldap_data [2],
+			  COLUMN_ILS_COMMENT, utf8_char_ldap_data [3],
+			  COLUMN_ILS_VERSION, utf8_remote_app,
+			  COLUMN_ILS_IP, ip ? ip : "",
+			  COLUMN_ILS_COLOR, color,
+			  -1);
+      gnomemeeting_threads_leave ();
+
+
+      /* Free some memory and put the variables in their initial state */
+      for (int i = 0 ; i < 6 ; i++) {
+	  
+	if (utf8_char_ldap_data [i])
+	  free (utf8_char_ldap_data [i]);
+
+	utf8_char_ldap_data [i] = NULL;
+      }
+
+      for (int i = 0 ; i < 3 ; i++) {
+
+	b_ldap_data [i] = false;
+      }
+	
+      g_free (ip);
+      g_free (color);
+      g_free (utf8_username);
+      g_free (utf8_remote_app);
+      g_free (utf8_callto);
+      ip = NULL;
+      color = NULL;
+      utf8_username = NULL;
+      utf8_remote_app = NULL;
+      utf8_callto = NULL;
+
+      gnomemeeting_threads_enter ();
+      g_object_unref (status_icon);
+      gnomemeeting_threads_leave ();
+    } /* end of for */
+
+    gnomemeeting_threads_enter ();
+    gtk_tree_view_set_model (GTK_TREE_VIEW (lwp->tree_view),
+			     GTK_TREE_MODEL (users_list_store));
+
+    if (num_users && num_users [1]) {
+
+      gnomemeeting_statusbar_push (lwp->statusbar, _("Search completed: %d user(s) listed on a total of %d user(s) from %s."), users_nbr, PMAX (atoi (num_users [1]), users_nbr), ldap_server);
+
+      g_strfreev (num_users);
+    }
+    else
+      gnomemeeting_statusbar_push (lwp->statusbar, _("Search completed: %d user(s) found on %s."), users_nbr, ldap_server);
+
+    gnomemeeting_threads_leave ();
   }
 
+
   if (no_error) {
 
     ldap_msgfree (res);
     ldap_unbind (ldap_connection);
   }
-  else {
-    
-    gnomemeeting_threads_enter ();
-    msg = g_strdup_printf (_("Failed to contact %s."), ldap_server);
-    gnomemeeting_statusbar_push (statusbar, msg);
-    g_free (msg);
-    gnomemeeting_threads_leave ();
-  } 
 }
